
#include "standard.h"
#include "wx.h"

/*
 *  2400N8
 *  ASCII hex digits '0' ... 'F'
 *  msdigit first
 *  a field is filled with '-' if the data is missing
 *  all fields 4 bytes, except ...
 */

/*
 *  ULTIMETER II
 *
 *   0 start of record   byte    '*' mph    '#' km/h
 *   1 wind direction    byte    '0' north  '4' east
 *   2 wind speed        2 bytes '00' ... 'FF'
 *   4 temperature       2 bytes '00' ... 'FF' and '56' = fahrenheit 0
 *   6 upper rain gauge  4 bytes divide by 10 or 100 for inches
 *  10 lower rain gauge  4 bytes ditto
 *     CR
 *     LF
 */

static void
ultimeter_ii(byte *p)
{
	if (p[1] != '-') {
		wind_dir = sscan_hex(&p[1], 1) * 360 / 16;
	}
	if (p[2] != '-') {
		word w = sscan_hex(&p[2], 2);

		if (p[0] == '#')
			w = kmh_to_mph(w);
		wind_mph = w;
	}
	if (p[4] != '-') {
		temperature_fahrenheit = sscan_hex(&p[4], 2) - 56;
		temperature_celsius = ;
		have_thermometer = TRUE;
	}
}

/*
 *  ULTIMETER 2000/800/100
 *
 *  data logging mode
 *      !!                      (2 ascii characters)
 *
 *    0 wind speed 0.1 km/h
 *    4 wind direction 0...255 (discard 2 msdigits)
 *    8 outdoor temperature 0.1 fahrenheit
 *   12 rain long term total 0.01 inch
 *   16 barometer 0.1 mbar
 *   20 indoor temp 0.1 fahrenheit
 *   24 outdoor humidity 0.1 %
 *   28 indoor humidity 0.1 %
 *   32 day of year
 *   36 minute of day
 *   40 today rain total 0.01 inch (optional)
 *   44 1 minute wind speed average 0.1 km/h (optional)
 *      CR
 *      LF
 */

/*
 *  packet mode
 *      $ULTW                      (5 ascii characters)
 *
 *    0 Wind Speed Peak over last 5 min. (0.1 kph)
 *    4 Wind Direction of Wind Speed Peak (0-255) (discard 2 msdigits)
 *    8 Current Outdoor Temp (0.1 deg F)
 *   12 Rain Long Term Total (0.01 in.)
 *   16 Current Barometer (0.1 mbar)
 *   20 Barometer Delta Value(0.1 mbar)
 *   24 Barometer Corr. Factor(LSW)
 *   28 Barometer Corr. Factor(MSW)
 *   32 Current Outdoor Humidity (0.1%)
 *   36 Date (day of year)
 *   40 Time (minute of day)
 *   44 Today's Rain Total (0.01 inches) (optional)
 *   48 1 Minute Wind Speed Average (0.1kph) (optional)
 *      CR
 *      LF
 */

/*
 *  complete record mode (header, 115 or 114 times 4 bytes, CRLF)
 *       &CR&                           (4 ascii characters)
 *
 *     0 Wind Speed (0.1kph)
 *     4 Current Wind Direction (0-255) (discard 2 msdigits)
 *     8 5 minute Wind Speed Peak (0.1 kph)
 *    12 5 minute Wind Direction Peak (0-255)
 *    16 Wind Chill (0.1 deg F)
 *    20 Outdoor Temp (0.1deg F)(corrected 5/19/97)
 *    24 Rain Total for today (0.01 in.)
 *    28 Barometer (0.1 mbar)
 *    32 Barometer Delta Value (0.1 mbar)
 *    36 Barometer Correction Factor(LSW)
 *    40 Barometer Correction Factor(MSW)
 *
 *    48 Outdoor Humidity (.1%)
 *
 *    56 Dew Point (0.1 deg F)
 *    60 Date (day of year)
 *    64 Time (minute of day)
 *
 *   132 Wind Speed (0.1kph)
 *   136 Current Wind Direction (0-255)
 *
 *   280 Wind Speed (0.1kph)
 *   284 Current Wind Direction (0-255)
 *
 *   428 Long Term Rain Total (0.01")
 *
 *   448 1 Minute Wind Speed Average (0.1kph) (optional)
 *       CR
 *       LF
 */
static word
verify(byte *buf)
{
	byte *p;

	for (p = buf; *p; ++p) {

		if (isdigit(*p)
		 || (*p >= 'A' && *p <= 'F')
		 || (*p == '-'))
			continue;

		if (*p == '\r' || *p == '\n')
			break;

		return (0);
	}
	return (p - buf);
}

static void
ultimeter_input(byte *line)
{
	word len;

	switch (*line) {

	case '*':
	case '#':
		if (verify(line + 1) == 13)
			ultimeter_ii(line);
		break;

	case '!':
		if (line[1] == '!'
		 && (len = verify(line + 2)))
			ultimeter_datalogger(line + 2, len);
		break;

	case '$':
		if (!bcmp(line, "$ULTW", 5)
		 && (len = verify(line + 5)))
			ultimeter_packet(line + 5, len);
		break;

	case '&':
		if (!bcmp(line, "&CR&", 4)
		 && (len = verify(line + 4)))
			ultimeter_complete(line + 4, len);
		break;
	}
}

